//-------------------------------------------------------------------------------------------------
//
// Oscilloscope Measurement and Keys
//
//-------------------------------------------------------------------------------------------------

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "stdlib.h"					// additional Lib
#include "string.h"					// additional Lib

#include "Scope.h"
#include "main.h"
#include "KS0108.h"					// definitions for KS108

#ifndef CTC1
#define CTC1 WGM12              	// for compatibility with ATmega
#endif

#define XTAL		16000000L    	// Crystal frequency in Hz
#define DEBOUNCE	100000L			// NEW: 100000 => 100000Hz = 10usec 

//-------------------------------------------------------------------------------------------------
// Initialize Key-Routies PC0-PC4 + Relais-Port PC5-PC7 and enable IRQ-Flag
//-------------------------------------------------------------------------------------------------

void KeyInit( void )				// 
{  
  TCCR1B = _BV(CTC1) + _BV(CS10);  	// CTC1=1 (=WGM12): clear timer1 (16 bit) on compare match, 
  									// CS10=1: no prescaler
  OCR1A  =  XTAL/DEBOUNCE;		   	// timer =>   IRQ = 10usec
  TIMSK  = _BV(OCIE1A);            	// enable Output Compare 1 overflow interrupt

// set Port C
  DDRC  = 0x00;                    	// PC0-PC5 for switch-input 
  									// PC6-PC7 N.C.- reserve switch
  PORTC = 0xff;                    	// for switch with pull-up enabled

// set Port A
  DDRA  = 0xf2;                    	// PA0 Input: ADC ! 
  									// PA1 Output for IRQ-Test
									// PA2, PA3: N.C. = Input
									// PA4-PA7 Output for relais

  PORTA = 0x00;                    	// PA0: ADC-Input 
  									// PA1: IRQ triggered Output 
									// PA2, PA3: open
									// PA4-PA7: Output = 0V

  delay_byte1 = 0;					// |Set bytes depending of selected scan-timing
  delay_byte2 = 0;					// |
  delay_byte3 = 0;					// | 0 0 0 0 => 10usec max. speed
  delay_byte4 = 0;					// |

  trigger_wait = 1;					// 1 = wait for trigger, 0 = no trigger
  
  display_voltage();				// initial display of voltage and setting of relais
  display_timing();					// initial display timing values
  display_modus();					// display of modus


  sei();                           	// enable interrupt
 return;	
}

//-------------------------------------------------------------------------------------------------
//  Initialize ADC of AVR-Processor
//-------------------------------------------------------------------------------------------------

void ADC_init(void)
{
	ADMUX  = 0b00100000; 				// PA0 -> ADC0, ADLAR=1 => bits left (8bit): ext VREF!
//	ADMUX  = 0b01100000; 				// PA0 -> ADC0, ADLAR=1 => bits left (8bit): TEST with +5V VREF!
//	ADCSRA = 0b11000100; 				// ADC prescaler at 16: 16MHz /16 = 1MHz ADC-Clock (~14us)
	ADCSRA = 0b11000011; 				// ADC prescaler at 8: 16MHz /8 = 2MHz ADC-Clock (~7us)
	scan_ready = 0;						// Start Scan
	read_done  = 0;						// 1.IRQ: Start ADC, 2.IRQ: Readout IRQ single shot ...
	trigger_level = 128;				// Start at center of voltage range (32 of 64 pixel)
	trigger_slope   = 0;				// start with + trigger
}

//-------------------------------------------------------------------------------------------------
// *** Interrupt start every 10 usec ***
//-------------------------------------------------------------------------------------------------

 	uint8_t y_position = 0;			// y-Position (Voltage 0..64 pixel)
 	uint8_t key_delay  = 0;			// counter for key delay

ISR (TIMER1_COMPA_vect)				// Timer/Counter1 Compare Match A
									// old: SIGNAL (SIG_OUTPUT_COMPARE1A)
{
	PORTA |= 0x02;					// Measure duration of IRQ: PA1 = +5 Volt if IRQ starts!

//-------------------------------------------------------------------------------------------------
// This routine reads ADC-value, waits for trigger and stores values
//-------------------------------------------------------------------------------------------------

 	uint8_t y_position_last = 0;									// previous y-Position

if (scan_ready == 0)										// allow readout & ADC: 0, 1...100 points
	{
	if (read_done == 0)										// allow readout of ADC
		{
				y_position = ADCH;							// use only 8 higher bits 2-9 
				read_done =1;								
															// & set 127 to center of display
					dataLcdBuffer[x_position] = y_position;	// save measured value in array

				if	(x_position < 101)						// 0 ...100; 0 is not displayed
					{										// if x-limit is reached, go to start
					x_position ++;
					}
				else 
					{
					x_position = 0;							// set start value for next ADC-scan
					scan_ready = 1;							// ADC-scan 0...100 finished
					}

//-------------------------------------------------------------------------------------------------
// Find trigger level + or -

	if(trigger_wait == 1)									// 1 = wait for trigger, 0 = no trigger
		{
		if(trigger_slope == 0)									// + trigger
			{
			if(x_position == 1)
				{
				y_position_last = y_position;					// rescue actual measured value
				if(y_position > trigger_level)
					x_position = 0;								// repeat if input > trigger level
				}

			if(x_position == 2 && y_position <= trigger_level)	// pos. trigger!
				if(!(y_position < y_position_last))
						x_position = 0;							// repeat if input <> trigger level
			}
		
		else													// - trigger
			{
			if(x_position == 1)
				{
				y_position_last = y_position;					// rescue actual measured value
				if(y_position < trigger_level)
					x_position = 0;								// repeat if input < trigger level
				}

			if(x_position == 2 && y_position >= trigger_level)	// neg. trigger!
				if(!(y_position < y_position_last))
						x_position = 0;							// repeat if input <> trigger level
				}
		
				

		}
	}

//-------------------------------------------------------------------------------------------------
//  Count down of timing during interupt

	if(!(count_S ==0))						// to save time in IRQ, only for range 100 us/Skt
	{

	if(delay_byte1 > 0)						// timing constant for scanning of signal
	delay_byte1 --;
	else {	

		if(delay_byte2 > 0)
		{
		delay_byte2 --;
		delay_byte1 = save_delay_byte1;	
		}
		else {

			if(delay_byte3 > 0)
			{
			delay_byte3 --;
			delay_byte2 = save_delay_byte2;
			}
			else {

				if(delay_byte4 > 0)
				{
				delay_byte4 --;
				delay_byte3 = save_delay_byte3;
				}
				else {	// all delay_byte = 0 ==> measure of next scan-Point allowed
					 delay_byte1 = save_delay_byte1;	// | Copy
					 delay_byte2 = save_delay_byte2;	// | Timing-Values
					 delay_byte3 = save_delay_byte3;	// | 20us...200min /SKT
					 delay_byte4 = save_delay_byte4; 	// | to 4x8Bit Down-Counter

					// start ADC (one AD-cycle needs 7usec at 2 MHz)
						ADCSRA |= (1 << ADSC); 	// start ADC and read at next IRQ while digitize = 0
						read_done = 0;			// allow ADC-readout

					 }									
				 }
			 }
		}

	}

	else
		{
		ADCSRA |= (1 << ADSC); 	// start ADC and read at next IRQ while digitize = 0
		read_done = 0;			// allow ADC-readout
		}
	 }

//-------------------------------------------------------------------------------------------------
// This routine reads pressed keys
//------------------------------------------------------------------------------------------------
/*
 Title:    	Debouncing 8 Keys
 Author:   	Peter Fleury <pfleury@gmx.ch> http://jump.to/fleury,
           	based on algorithm of Peter Dannegger <danni@specs.de>

 			Modified: Karl-Anton Dichtel for Scope-Hardware

 Description of key-management:
 Demonstrates debouncing 8 keys. The state of the eight keys is sampled
 four times using a timer interrupt. If a key is pressed longer than
 four seconds, the corresponding bit in the key_press global variable is set.
 The main loop checks if a bit is set in this global variable and resets 
 the bit.

 Module global variable, informs the main programm when a key is pressed.
 This variable must be declared 'volatile, since it is accessed from
 both interrupt und main loop.
*/
//------------------------------------------------------------------------------------------------

  static uint8_t key_state;		// debounced and inverted key state:
  static uint8_t ct0, ct1;  	// holds two bit counter for each key
  uint8_t i;					// local loop

  		/*
   		* read current state of keys (active-low),
   		* clear corresponding bit in i when key has changed
   		*/
  		i = key_state ^ ~PINC;   		// Port C (only PC0-PC5 used): key changed ?
  
  		/* 
   		* ct0 and ct1 form a two bit counter for each key,  
   		* where ct0 holds LSB and ct1 holds MSB
   		* After a key is pressed longer than four times the
   		* sampling period, the corresponding bit in key_state is set
   		*/

	key_delay++;				// decrement with IRQ (10usec)
	if(key_delay == 255)	// scan key only avery 255 IRQ's = 2.55msec
		{
		key_delay =0;

  		ct0 = ~( ct0 & i );			// reset or count ct0
  		ct1 = (ct0 ^ ct1) & i;	    // reset or count ct1  
  		i &= ct0 & ct1;			    // count until roll over ?
  		key_state ^= i;			    // then toggle debounced state
  
  		/*
   		* To notify main program of pressed key, the correspondig bit
   		* in global variable key_press is set.
   		* The main loop needs to clear this bit
   		*/
  		key_press |= key_state & i;	// 0->1: key press detect
  		}

//-------------------------------------------------------------------------------------------------

	if(time_line > 0)				// make Scan-display delay with IRQ // 10us * 50.000 = 0,5 sec
		time_line --;				// unsigned short int =0..65 535


 PORTA &= 0xfd;						// TEST: PA1 = 0 Volt if IRQ is finished!

}		// End of ISR


//
//-------------------------------------------------------------------------------------------------
// Key-Routines called from main programm
//-------------------------------------------------------------------------------------------------

volatile uint8_t count_V  = 6;				// up and down per key (Start: 500mV/all relais off) !
volatile uint8_t count_S  = 0;				// up and down per key (Start: 20uSec) !
char AsciiZeichen[4];						// clear display
uint16_t Zahl = 0;							// used to convert
uint8_t set_hw;								// Set relais and timing
char AsciiZeichen[4];						// display text
uint8_t i;									// Index for loops
char Display_Byte[4] = "    ";				// 4 Digits: Time or Range or Modus

void KeyScan( void )	
{

//-------------------------------------------------------------------------------------------------

// Key-Routines Key 0 & 3 - Input Voltage Range: 0...6 => 5mV ...500mV

    if (key_press & _BV(0) )						// Bit 0 set?
    {	
		if(trigger_level < 251 && mod_sel == 3)		// Trigger level +4
			{
			trigger_level = trigger_level+4;
			 display_modus();						// display of modus
			}
		else
			{
        	if (count_V < (uint8_t) 6)				// Input voltage range
				{
				count_V ++;							// next
				}
			}

		display_voltage();							// display of voltage and setting of reais

        key_press = 0;
    }

    else if ( key_press & _BV(3) )  				// Bit 3 set?
    {
		if(trigger_level > 3 && mod_sel == 3)		// Trigger level -4
			{
			trigger_level = trigger_level-4;
			display_modus();						// display of modus
			}

		else
			{
        	if (count_V > (uint8_t) 0)
			{
			count_V --;

			}
		}

		display_voltage();							// display of voltage and setting of relais

        key_press = 0;
    }

//-------------------------------------------------------------------------------------------------

// Key-Routines Key 4 & 1 - Timing

	if (key_press & _BV(4) )									// Bit 4 set?
    {

        if (count_S < (uint8_t) 24)								// range 100us ... 200min
		{
		count_S ++;
		}
        
		display_timing();										// display timing values

        key_press = 0;

		// change immedially
		save_delay_byte1 = pgm_read_byte(&timing[count_S][6]);	// | Read and store
		save_delay_byte2 = pgm_read_byte(&timing[count_S][7]);	// | Timing-Range Table:
		save_delay_byte3 = pgm_read_byte(&timing[count_S][8]);	// | 200us...10msec /SKT
		save_delay_byte4 = pgm_read_byte(&timing[count_S][9]);	// | and store it
    }

    else if ( key_press & _BV(1) )  	// Bit 1
    {

        if (count_S > (uint8_t) 0)
		{
		count_S --;

		}

		display_timing();										// display timing values
        
        key_press = 0;

		// change immedially
		save_delay_byte1 = pgm_read_byte(&timing[count_S][6]);	// | Read and store
		save_delay_byte2 = pgm_read_byte(&timing[count_S][7]);	// | Timing-Range Table:
		save_delay_byte3 = pgm_read_byte(&timing[count_S][8]);	// | 200us...10msec /SKT
		save_delay_byte4 = pgm_read_byte(&timing[count_S][9]);	// | and store it	
    }


//-------------------------------------------------------------------------------------------------

// Key-Routines Key 3 - Modus (default: 0 => Auto)

	if (key_press & _BV(2) )						// Bit 2 set?
    {
		GLCD_GoTo(104,7);
		GLCD_WriteString ("    ");					// Clear old figures

	// key (Modus) pressed, increment counter Modus
        if (mod_sel < (uint8_t) 3)					// => 4 Modi to select
		{
		mod_sel ++;
		display_mode = 3;							// clear display, if mode is changed
		}
         else if ( mod_sel == 3 ) 					// => 4 Modi to select
		{
		mod_sel = 0;
		display_mode = 3;							// clear display, if mode is changed
		}

		 display_modus();							// display of actual modus
		 	
//----------------------------------------------		 					

	// show actual AC/DC setting
		if(mod_sel == 0)
			{
			show_AC_DC();							// set AC/DC-Input-relais and display it
			}

	// no trigger in CONT-Modus
		if(mod_sel == 1)
			trigger_wait = 0;						// no trigger
		else
			trigger_wait = 1;						// wait for trigger


	// single sweep in SING-Modus
		if(mod_sel == 2)
			{
			trigger_wait = 0;						// no trigger
			single_scan  = 0;						// 0 = wait, start single scan by pressing button
			}										// 1 = start one cycle => 0

	 // show Triggerlevel (0...32) on Display
		if(mod_sel == 3)
			{
			GLCD_GoTo(116,7);						// (Column-Pixel,Line)
			itoa (trigger_level/4, AsciiZeichen, 10);
			GLCD_WriteString (AsciiZeichen);		// (char charToWrite)

			show_trigger_slope();					// show on Display +/-
			}

        key_press = 0;

	}

//-------------------------------------------------------------------------------------------------

// Key-Routines Key 5 - Toggle (depending of selected Modus on Key 3)

	if (key_press & _BV(5) )				// Bit 5 set?
  		{
	// only Modus "AUTO"
			if(mod_sel == 0)
				{
				AC_DC ^= 0x80;				// toggle AC/DC
				set_AC_DC();				// set AC/DC-Input-relais and display it
				}

	// only Modus "SING"
			if(mod_sel == 2 && single_scan == 0)
				{
				display_mode = 3;			// clear display, if mode is changed
				scan_ready = 0;				// allow next Scan of ADC and readout
				trigger_wait = 0;			// no trigger, start directly measuring
				single_scan  = 1;			// | start single scan by pressing button
											// | and allow display of "one scan"
				GLCD_GoTo(104,7);			// (Column-Pixel,Line)
				GLCD_WriteString (" Run");	// display " Run"
				}

	// only Modus "Trig"
			if(mod_sel == 3)
				{
				trigger_slope ^= 0x01;		// toggle +/- 

				show_trigger_slope();		// show on Display
				}

        	key_press = 0;
		}

//-------------------------------------------------------------------------------------------------

  return;

}			// End of KeyScan


//-------------------------------------------------------------------------------------------------
// Subroutines starting here
//-------------------------------------------------------------------------------------------------

	// update Display of Voltage-Range (5-500mV) and info-field - only if key is pressed

	void display_voltage(void)
	{
		GLCD_GoTo(104,1);
		GLCD_WriteString ("   ");								// Clear old range-figures

		Zahl = pgm_read_word(&gain_display[count_V]);			// out off Voltage-Range Table 5...500mV
		itoa (Zahl, AsciiZeichen, 10);							// AVR-Bibliothek (Integer to ASCII)
		GLCD_GoTo(104+(6*(3-(strlen(AsciiZeichen)))),1);		// (Start string: Column-Pixel,Line)
		GLCD_WriteString (AsciiZeichen);

		set_hw = pgm_read_byte(&gain_relais[count_V]);			// read Voltage-Range-Relais Table

		PORTA |= set_hw;										// set relais
		set_hw |= 0x80;											// dont change bit 7 (AC/DC-relais)
  		PORTA &= set_hw;										// or clear relais
        

		// show Triggerlevel (0...32) on Display

		if(mod_sel == 3)
		{
		GLCD_GoTo(116,7);										// (Column-Pixel,Line)
		itoa (trigger_level/4, AsciiZeichen, 10);
		GLCD_WriteString (AsciiZeichen);						// (char charToWrite)

		void show_trigger_slope();							// display + or - trigger slope
		}
	}
//-------------------------------------------------------------------------------------------------

	// update Display of Timing-Range (10us-200 min/Skt) - only at change

	void display_timing(void)
	{
	GLCD_GoTo(104,3);											// (Column-Pixel,Line)

	   	for (i=0; i < 3; i++)
	   	{
		Display_Byte[i] = pgm_read_byte(&timing[count_S][i]);	// out off Timing-Range Table 20us...200min
		GLCD_WriteChar(Display_Byte[i]);	
		}
		strcpy (Display_Byte, "   ");							// clear

	GLCD_GoTo(104,4);											// (Column-Pixel,Line)

	   	for (i=0; i < 3; i++)
	   	{
		Display_Byte[i] = pgm_read_byte(&timing[count_S][i+3]);	// out off Timing-Range Table 20us...200min
		GLCD_WriteChar(Display_Byte[i]);	
		}
	}
//-------------------------------------------------------------------------------------------------

	// Display Modus of Scope: Auto, Cont....

	void display_modus(void)
	{
	GLCD_GoTo(104,6);											// (Column-Pixel,Line)

	   	for (i=0; i < 4; i++)
	   	{
		Display_Byte[i] = pgm_read_byte(&modus[mod_sel][i]);	// selected Modus
		GLCD_WriteChar(Display_Byte[i]);	
		}
	}
//------------------------------------------------------------------------------------------------- 

	// set AC/DC-input-relais and display it

		void set_AC_DC(void)
			{
			if(AC_DC == 0)						// 0 => AC  
				{
				PORTA &= 0x7f;					// AC/DC-relais off
				GLCD_GoTo(116,7);				// (Column-Pixel,Line)
				GLCD_WriteString ("AC");		// display "AC"
				}

			else								// 1 => DC
				{
				PORTA |= 0x80;					// AC/DC-relais on
				GLCD_GoTo(116,7);
				GLCD_WriteString ("DC");		// display "DC"
				}
			}

//------------------------------------------------------------------------------------------------- 


	// show AC/DC at display

		void show_AC_DC(void)
			{
			GLCD_GoTo(116,7);				// (Column-Pixel,Line)

			if(AC_DC == 0)						// 0 => AC;  1 => DC
				GLCD_WriteString ("AC");		// display "AC"

			else
				GLCD_WriteString ("DC");		// display "DC"
			}

//-------------------------------------------------------------------------------------------------

	// display trigger slope + or -

		void show_trigger_slope(void)
			{
			GLCD_GoTo(104,7);			// (Column-Pixel,Line)

			if(trigger_slope == 0)
				GLCD_WriteString ("+");		// (char charToWrite)

			else
				GLCD_WriteString ("-");		// (char charToWrite)
			}

//-------------------------------------------------------------------------------------------------
